@@ -39,13 +39,14 @@
 #include "slog.h"
 #include "global.h"
 
-#define DEFAULT_REKEY	       		3600
-#define DEFAULT_KEEPALIVE      		60	// one keepalive/minute (it's just 8 bytes...)
-#define DEFAULT_UDPPORT	       		655	// same as tinc, conflicts would be rare
-#define DEFAULT_MTU	       		1500	// let's ether-net
-#define DEFAULT_MAX_RETRY      		3600	// retry at least this often
-#define DEFAULT_MAX_TTL      		60	// packets expire after this many seconds
-#define DEFAULT_MAX_QUEUE      		512	// never queue more than this many packets
+#define DEFAULT_REKEY			3607	// interval between rekeys
+#define DEFAULT_RESEED			3613	// interval between rng reseeds
+#define DEFAULT_KEEPALIVE		60	// one keepalive/minute (it's just 8 bytes...)
+#define DEFAULT_UDPPORT			655	// same as tinc, conflicts would be rare
+#define DEFAULT_MTU			1500	// let's ether-net
+#define DEFAULT_MAX_RETRY		3600	// retry at least this often
+#define DEFAULT_MAX_TTL			60	// packets expire after this many seconds
+#define DEFAULT_MAX_QUEUE		512	// never queue more than this many packets
  
 #define DEFAULT_DNS_TIMEOUT_FACTOR	8.F	// initial retry timeout multiple
 #define DEFAULT_DNS_SEND_INTERVAL	.01F	// minimum send interval
@@ -59,6 +60,7 @@ enum
   PROT_TCPv4  = 0x04, // tcp over ipv4 (server)
   PROT_ICMPv4 = 0x08, // icmp over ipv4
   PROT_DNSv4  = 0x10, // dns tunnel ipv4 (server)
+  PROT_ALL    = 0x1f
 };
 
 #define PROT_RELIABLE (PROT_TCPv4 | PROT_DNSv4)
@@ -123,6 +125,8 @@ struct configuration
   node_vector nodes;
   conf_node default_node;
   conf_node *thisnode;
+  char *seed_dev;   // the randomd evice to use for seeding
+  double reseed;    // the interval between additional seeds
   int mtu;          // the mtu used for outgoing tunnel packets
   int nfmark;       // the SO_MARK // netfilter mark // fwmark
   double rekey;     // rekey interval
@@ -133,6 +137,9 @@ struct configuration
   RSA *rsa_key;     // our private rsa key
   loglevel llevel;
   u8 ip_proto;      // the ip protocol to use
+  uid_t change_uid; // the uid of the user to switch to, or 0
+  gid_t change_gid; // the gid of the user to switch to, or 0
+  char *change_root;// the path to chroot to, "/" == anonymous
 #if ENABLE_ICMP
   u8 icmp_type;     // the icmp type for the icmp-protocol
 #endif
@@ -151,6 +158,7 @@ struct configuration
 
 #if ENABLE_DNS
   char *dns_forw_host;
+  bool dns_case_preserving;
   u16 dns_forw_port;
   float dns_timeout_factor;
   float dns_send_interval;
@@ -164,7 +172,7 @@ struct configuration
 
   // create a filename from string, replacing %s by the nodename
   // and using relative paths under confbase.
-  char *config_filename (const char *name, const char *dflt);
+  char *config_filename (const char *name, const char *dflt = 0);
 
   void print ();
 
@@ -184,6 +192,7 @@ struct configuration_parser
 
   configuration_parser (configuration &conf, bool need_keys, int argc, char **argv);
 
+  void parse_file (const char *fname);
   const char *parse_line (char *line);
   void parse_argv ();
 };
